<?php
declare(strict_types=1);

namespace Gitamine\Tests\Handler;

use App\Prooph\SynchronousQueryBus;
use Gitamine\Domain\Directory;
use Gitamine\Domain\Event;
use Gitamine\Domain\Plugin;
use Gitamine\Handler\GetConfiguratedPluginsQueryHandler;
use Gitamine\Infrastructure\GitamineConfig;
use Gitamine\Query\GetConfiguratedPluginsQuery;
use Gitamine\Query\GetProjectDirectoryQuery;
use Hamcrest\Matchers;
use PHPUnit\Framework\TestCase;

/**
 * Class GetConfiguratedPluginsQueryHandlerTest
 *
 * @package Gitamine\Tests\Handler
 */
class GetConfiguratedPluginsQueryHandlerTest extends TestCase
{
    public function testGetConfiguratedPlugins()
    {
        $dir = '/';

        $bus      = \Mockery::mock(SynchronousQueryBus::class);
        $gitamine = \Mockery::mock(GitamineConfig::class);

        $bus->shouldReceive('dispatch')
            ->once()
            ->with(Matchers::equalTo(new GetProjectDirectoryQuery()))
            ->andReturn($dir);

        $gitamine->shouldReceive('getPluginList')
                 ->once()
                 ->with(Matchers::equalTo(new Directory($dir)), Matchers::equalTo(new Event(Event::PRE_COMMIT)))
                 ->andReturn([new Plugin('test')]);

        $handler = new GetConfiguratedPluginsQueryHandler($bus, $gitamine);

        self::assertEquals(
            ['test'],
            $handler(new GetConfiguratedPluginsQuery('pre-commit'))
        );
    }
}
